<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Cache.php,v 1.1 2005/04/04 11:50:57 mike Exp $
 */

/**
 * Requires Gadget_Provider
 */
class_exists('Gadget_Provider') or require_once 'Gadget/Povider.php';

/**
 * Gadget_Provider_Cache
 *
 * Gadget's default cache provider
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @since       1.0
 */
class Gadget_Provider_Cache extends Gadget_Provider
{
    /**
     * Provider
     * 
     * @var string
     */
    var $provider = PROVIDER_CACHE;
    
    /**
     * Cache
     * 
     * @var object
     */
    var $cache;
    
    /**
     * Cache Config
     * 
     * @var array
     */
    var $conf;
    
    /**
     * Page
     * 
     * @var string
     */
    var $page;
    
    /**
     * Current User
     * 
     * @var int
     */
    var $user = 0;
    
    /**
     * Ident
     * 
     * @var string
     */
    var $ident;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init()
    {
        $this->page = $_SERVER['PHP_SELF'];
        $this->conf = Gadget::getConf('Cache');
        $this->user = Gadget::currentUser();
        
        if (!empty($this->conf[$this->page])) {
            class_exists('Cache') or require_once 'Cache.php';
            $this->cache = &new Cache($this->conf['container'], $this->conf);
            $this->ident = $this->cache->generateId(
                array(
                    $this->page,
                    $this->user,
                    Gadget::getQ(),
                    Gadget::getPref(),
                )
            );
            $this->time = $this->conf[$this->page];
        }
    }
    
    /**
     * Get
     * 
     * @access  public
     * @return  string
     */
    function get()
    {
        $cacheData = null;
        if (!empty($this->conf[$this->page])) {
            if ($this->isCachableRequest()) {
                if ($cacheData = $this->cache->get($this->ident, $this->page)) {
                    Gadget::log('CACHE', '[HIT]');
                } else {
                    Gadget::log('CACHE', '[MISS]');
                }
            } else {
                $this->cache->remove($this->ident, $this->page);
            }
        }
        return $cacheData;
    }
    
    /**
     * Set
     * 
     * @access  public
     * @return  bool
     * @param   string
     */
    function set($cacheData = null)
    {
        if (!empty($this->conf[$this->page])) {
            return $this->cache->save(
                $this->ident, 
                isset($cacheData) ? $cacheData : ob_get_contents(), 
                $this->conf[$this->page], 
                $this->page
            );
        }
        return true;
    }
    
    /**
     * Is Cachable Request
     * 
     * @access  public
     * @return  bool
     */
    function isCachableRequest()
    {
        if (isset($_SERVER['REQUEST_METHOD'])) {
            return  $_SERVER['REQUEST_METHOD'] == 'HEAD' || 
                    $_SERVER['REQUEST_METHOD'] == 'GET';
        }
        return false;
    }
    
}

?>
