<?php

/**
 * Gadget Webdevelopment Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Search
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Fulltext.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/**
 * Gadget_Search_Fulltext
 * 
 * @package     Gadget
 * @subpackage  Gadget_Search
 * @since       1.0
 */
class Gadget_Search_Fulltext extends Gadget_Object
{
    /**
     * Foreign ID
     * 
     * @var string
     */
    var $foreignId  = 'id';
    
    /**
     * Search Columns
     * 
     * @var array
     */
    var $cols  = array();
    
    /**
     * Table(s)
     * 
     * @var string
     */
    var $table = '';
    
    /**
     * Where Condition
     * 
     * @var string
     */
    var $where = '';
    
    /**
     * Ident String
     * 
     * @var string
     */
    var $ident = '';
    
    /**
     * Search String
     * 
     * @var string
     */
    var $search = '';
    
    /**
     * Boolean Search
     * 
     * @var int
     */
    var $bool;
    
    /**
     * Query
     * 
     * @var string
     */
    var $query = '
        INSERT INTO searches
            (ident, foreignid, rank)
        SELECT
            "%s",
            %s,
            MATCH(%s) AGAINST ("%s" %s) AS Gadget_Search_RANK
        FROM
            %s
        WHERE
            MATCH(%s) AGAINST ("%s" %s)
            %s
        ORDER BY
            Gadget_Search_RANK DESC
    ';
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     * @param   array   $options
     */
    function onAfterConstruct($options)
    {
        $this->search and $this->ident = $this->generateIdent($options);
    }
    
    /**
     * Generate Ident
     * 
     * @access  public
     * @return  string
     * @param   mixed   $idents
     */
    function generateIdent($idents)
    {
        return md5(serialize($idents));
    }
    
    /**
     * Do Search
     * 
     * @access  public
     * @return  mixed
     */
    function doSearch()
    {
        $search = Gadget::DAO('searches');
        $fields = implode(', ', $this->cols);
        $string = $search->escape($this->search);
        
        $squery = sprintf(
            $this->query,
            $this->ident,
            $this->foreignId,
            $fields,
            $string,
            $this->bool ? 'IN BOOLEAN MODE' : '',
            $this->table,
            $fields,
            $string,
            $this->bool ? 'IN BOOLEAN MODE' : '',
            $this->where ? ' AND '. $this->where : ''
        );
        
        return $search->query($squery);
    }
    
    /**
     * Result Exists
     * 
     * @access  public
     * @return  int
     * @param   int     $expires
     */
    function resultExists($expires = 1200)
    {
        $search = Gadget::DAO('searches');
        $search->ident = $this->ident;
        $search->whereAdd('tstamp > '. $search->timestamp(TIME - $expires*1000));
        return $search->count();
    }
    
    /**
     * Clear Results
     * 
     * @access  public
     * @return  bool
     * @param   int     $expires
     */
    function clearResults($expires = 1200)
    {
        $search = Gadget::DAO('searches');
        $search->ident = $this->ident;
        $search->whereAdd('tstamp < '. $search->timestamp(TIME - $expires*1000));
        return $search->delete(true);
    }
    
}

?>
