<?php

/**
 * Gadget Webdevelopment Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Action.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/**
 * Requires Gadget_Object
 */
class_exists('Gadget_Object') or require_once 'Gadget/Object.php';

/**
 * Registered Actions
 * @ignore
 */
$GLOBALS['_Gadget']['Template']['Actions'] = array(
    'clean'     => array('Gadget_Template_Action_clean',    null),
    'var'       => array('Gadget_Template_Action_var',      null),
    'gadget'    => array('Gadget_Template_Action_gadget',   null),
    'echo'      => array('Gadget_Template_Action_echo',     null),
    'gettext'   => array('Gadget_Template_Action_gettext',  null),
    'if'        => array('Gadget_Template_Action_if',       null),
    'while'     => array('Gadget_Template_Action_while',    null),
    'foreach'   => array('Gadget_Template_Action_foreach',  null),
    'else'      => array('Gadget_Template_Action_else',     null),
    'elseif'    => array('Gadget_Template_Action_elseif',   null),
    'end'       => array('Gadget_Template_Action_end',      null),
    'b64'       => array('Gadget_Template_Action_b64',      null),
);

/** 
 * Gadget_Template_Action
 * 
 * Base class for Gadget_Template actions.  A template action 
 * translates a specific part of template code into PHP code.
 * That means e.g. Gadget_Template_Action_if 
 * translates {if $foo} into <?php if ($foo) { ?>.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @since       1.0
 */
class Gadget_Template_Action extends Gadget_Object
{
    /**
     * Name
     * 
     * @var string
     */
    var $name;
    
    /**
     * Callback
     * 
     * @var string
     */
    var $callback = 'perform';
    
    /**
     * Composed Pattern
     * 
     * @var string
     */
    var $composedPattern;
    
    /**
     * Standard Pattern
     * 
     * @var string
     */
    var $standardPattern;
    
    /**
     * Recursive Pattern
     * 
     * @var string
     */
    var $recursivePattern;
    
    /**
     * Pattern Modifier
     * 
     * @var string
     */
    var $patternModifier = 'i';
    
    /**
     * PCRE Delimiter
     * 
     * @var string
     */
    var $pcreDeleimiter = '/';
    
    /**
     * PCRE Recursion
     * 
     * @var string
     */
    var $pcreRecursion;
    
    /**
     * Quoted Chars
     * 
     * @var array
     */
    var $quotedChars;
    
    /**
     * Phpize
     * 
     * @var bool
     */
    var $phpize = true;
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
        if ($this->standardPattern) {
            $this->composedPattern = 
                $this->pcreDelimiter          .
                $this->quotedChars['start']   .
                $this->standardPattern        .
                $this->quotedChars['end']     .
                $this->pcreDelimiter          .
                $this->patternModifier
            ;
        } elseif ($this->recursivePattern) {
            $this->composedPattern = 
                $this->pcreDelimiter          .
                $this->quotedChars['start']   .
                $this->recursivePattern       .
                $this->pcreRecursion          .
                $this->quotedChars['end']     .
                $this->pcreDelimiter          .
                $this->patternModifier
            ;
        }
    }
    
    /**
     * Perform
     * 
     * @access  public
     * @return  string
     * @param   array   $args
     */
    function perform($args)
    {
        if ($this->phpize) {
            return '<?php '. $args[2] .' ?>';
        }
        return $args[2];
    }
    
    /**
     * Parse Arg
     * 
     * @access  protected
     * @return  string
     * @param   string  $arg
     */
    function parseArg($arg)
    {
        if ($this->quotedChars['var'] == '\\$') {
            return $arg;
        }
        return preg_replace(
            '/(?<!'. $this->quotedChars['escape'] .')'. 
                $this->quotedChars['var'] .'/', '$', $arg);
    }
    
    /**
     * Register Action
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $name
     * @param   string  $class
     * @param   string  $file
     */
    function registerAction($name, $class, $file = null)
    {
        $GLOBALS['_Gadget']['Template']['Actions'][$name] = 
            array($class, $file);
    }
    
    /**
     * Unregister Action
     * 
     * @static
     * @access  public
     * @return  void
     * @param   string  $name
     */
    function unregisterAction($name)
    {
        unset($GLOBALS['_Gadget']['Template']['Actions'][$name]);
    }
    
    /**
     * Load Actions
     * 
     * @static
     * @access  public
     * @return  void
     * @param   array   $actions
     * @param   array   $options
     */
    function loadActions(&$actions, $options)
    {
        $actions = array();
        foreach ($GLOBALS['_Gadget']['Template']['Actions'] as $name => $action) {
            $actions[$name] = &Gadget::persistentObject(
                $action[0], $options, $action[1]);
        }
    }
}

?>
