<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Node.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/**
 * Gadget_Tree_Node
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Node
{
    /**
     * Name
     * 
     * @var string 
     */
    var $name;

    /**
     * Data
     * 
     * @var string 
     */
    var $data;

    /**
     * Meta
     * 
     * @var array 
     */
    var $meta = array();

    /**
     * Parent Node
     * 
     * @var object 
     */
    var $parent;

    /**
     * Childs
     * 
     * @var array 
     */
    var $childs = array();

    /**
     * Following Node
     * 
     * @var object 
     */
    var $following;

    /**
     * Preceding Node
     * 
     * @var object 
     */
    var $preceding;

    /**
     * Name of Class
     * 
     * @var string 
     */
    var $self = __CLASS__;

    /**
     * Full Path
     * 
     * @var string
     */
    var $fullPath;
    
    /**
     * Gadget_Tree_Node
     * 
     * @access public 
     */
    function Gadget_Tree_Node($name)
    {
        $this->name = $name;
    }

    /**
     * Get Name
     * 
     * @access  public
     * @return  string
     */
    function getName()
    {
        return $this->name;
    }
    
    /**
     * Reset Links
     * 
     * @access  public 
     * @return  void 
     */
    function resetLinks()
    {
        unset($this->parent, $this->following, $this->preceding);
        $this->parent = null;
        $this->following = null;
        $this->preceding = null;
    }

    /**
     * Set Parent
     * 
     * @access  public 
     * @return  bool 
     */
    function setParent(&$parentNode)
    {
        if ($isNode = is_a($parentNode, 'Gadget_Tree_Node')) {
            $this->parent = &$parentNode;
        }
        return $isNode;
    }

    /**
     * Get Data
     * 
     * @access  public
     * @return  mixed
     */
    function &getData()
    {
        return $this->data;
    }
    
    /**
     * Add Data
     * 
     * @access  public 
     * @return  void 
     */
    function addData($data)
    {
        $this->data .= $data;
    }

    /**
     * Set Data
     * 
     * @access  public 
     * @return  void 
     */
    function setData($data)
    {
        $this->data = $data;
    }

    /**
     * Get Meta
     * 
     * @access  public
     * @return  mixed
     */
    function getMeta($meta = null)
    {
        return isset($meta) 
            ? (isset($this->meta[$meta]) ? $this->meta[$meta] : null) 
            : $this->meta;
    }
    
    /**
     * Set Meta
     * 
     * @access  public 
     * @return  void 
     */
    function setMeta($meta = array())
    {
        $this->meta = (array) $meta;
    }

    /**
     * Add Meta
     * 
     * @access  public 
     * @return  mixed 
     */
    function addMeta($meta, $value = null)
    {
        if (is_array($meta)) {
            $this->meta = array_merge($this->meta, $meta);
        } else {
            $this->meta[$meta] = $value;
        }
    }

    /**
     * Add New Child
     * 
     * @access  public 
     * @return  mixed 
     */
    function &addNewChild($name)
    {
        $class = $this->self;
        $child = &new $class($name);
        return $this->addChild($child);
    }

    /**
     * Add Child
     * 
     * @access  public 
     * @return  mixed 
     */
    function &addChild(&$child)
    {
        return $this->setChild($child, $this->count());
    }

    /**
     * Set Child
     * 
     * @access  protected
     * @return  object
     * @param   object  $child
     * @param   int     $pos
     */
    function &setChild(&$child, $pos)
    {
        $pre = $pos - 1;
        $fol = $pos + 1;

        if (isset($this->childs[$pre])) {
            $child->linkPreceding($this->childs[$pre]);
        }

        if (isset($this->childs[$fol])) {
            $child->linkFollowing($this->childs[$fol]);
        }

        $this->childs[$pos] = &$child;
        $child->parent = &$this;

        return $child;
    }

    /**
     * Replace Child
     * 
     * Replace a direct child of this node with another node.  Returns PEAR_Error
     * GADGET_TREE_E_CHILD_NOT_FOUND if there's no child at the specified position.
     * 
     * @access  public 
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   mixed   $from Either int position of the child or the child to replace.
     * @param   object  $to The Gadget_Tree_Node that should replace to other child.
     */
    function replaceChild(&$from, &$to)
    {
        if ($isNode = is_a($from, $this->self)) {
            $pos = $from->position();
        } else {
            $pos = (int) $from;
        }
        if (!array_key_exists($pos, $this->childs) ||
                ($isNode && ($this->childs[$pos] !== $from))) {
            return Gadget_Tree::raiseError(
                "Couldn't find a child node at position $pos.",
                GADGET_TREE_E_CHILD_NOT_FOUND
            );
        }
        $this->setChild($to, $pos);
        return true;
    }

    /**
     * Insert Child
     * 
     * @todo    implement
     * @access  public 
     * @return  object 
     */
    function &insertChild(&$newChild, $pos = 0)
    {
        for($i = $this->count(); $i >= $pos; $i--) {
            $this->childs[$i + 1] = &$this->childs[$i];
        }
        $this->setChild($newChild, $pos);
        return $newChild;
    }

    /**
     * Remove Child
     * 
     * @access  public 
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   object  $child The Gadget_Tree_Node object to remove.
     */
    function removeChild(&$child)
    {
        $pos = $child->position();
        if (isset($this->childs[$pos]) && $this->childs[$pos] === $child) {
            $cnt = $this->count();
            $this->childs[$pos - 1]->linkFollowing($this->childs[$pos + 1]);
            for ($i = $pos; $i <= $cnt; $i++) {
                $this->childs[$i] = &$this->childs[$i + 1];
            }
            return true;
        }
        return Gadget_Tree::raiseError(
            "Couldn't find child node at position $pos.",
            GADGET_TREE_E_CHILD_NOT_FOUND
        );
    }

    /**
     * Link Following
     * 
     * @access  protected 
     * @return  void 
     */
    function linkFollowing(&$node)
    {
        $this->following = &$node;
        $node->preceding = &$this;
    }

    /**
     * Link Preceding
     * 
     * @access  protected 
     * @return  void 
     */
    function linkPreceding(&$node)
    {
        $this->preceding = &$node;
        $node->following = &$this;
    }

    /**
     * Get Children
     * 
     * This returns the whole tree below this node.
     * Use child() if you want to get the first level of children.
     * 
     * @access  public 
     * @return  array
     */
    function &getChildren()
    {
        return $this->childs;
    }

    /**
     * To Array
     * 
     * @access  public 
     * @return  mixed 
     */
    function &toArray()
    {
        Gadget_Tree::unfold($nodes = array(&$this), $array);
        return $array;
    }

    /**
     * Ancestor
     * 
     * Returns an array of parent nodes.
     * 
     * @access  public 
     * @return  array 
     */
    function &ancestor()
    {
        return Gadget_Tree::walkLink($this, 'parent');
    }

    /**
     * Ancestor or Self
     * 
     * Returns array of the node itself and parent nodes.
     * 
     * @see     ancestor()
     * @access  public 
     * @return  array 
     */
    function &ancestorOrSelf()
    {
        return Gadget_Tree::walkLink($this, 'parent', true);
    }

    /**
     * Child
     * 
     * @access  public 
     * @return  array 
     */
    function &child()
    {
        return $this->childs;
    }

    /**
     * Child At
     * 
     * @access  public
     * @return  object
     * @param   int     $pos
     */
    function &childAt($pos)
    {
        return $this->childs[$pos];
    }
    
    /**
     * Descendant
     * 
     * @access  public 
     * @return  array 
     */
    function &descendant()
    {
        Gadget_Tree::unfold($this->childs, $descendant = array());
        return $descendant;
    }

    /**
     * Descendant or Self
     * 
     * @access  public 
     * @return  array 
     */
    function &descendantOrSelf()
    {
        Gadget_Tree::unfold($this->childs, $descendant = array(&$this));
        return $descendant;
    }

    /**
     * Following
     * 
     * @access  public 
     * @return  array 
     */
    function &following()
    {
        if (!$this->following) {
            return array();
        }
        Gadget_Tree::unfold($this->followingSibling(), $following = array());
        return $following;
    }

    /**
     * Following Sibling
     * 
     * @access  public 
     * @return  array 
     */
    function &followingSibling()
    {
        return Gadget_Tree::walkLink($this, 'following');
    }

    /**
     * Parent
     * 
     * @access  public 
     * @return  object 
     */
    function &parent()
    {
        return $this->parent;
    }

    /**
     * Preceding
     * 
     * @todo    implement
     * @access  public 
     * @return  mixed 
     */
    function &preceding()
    {
        if (!$this->preceding) {
            return array();
        }
        Gadget_Tree::unfold($this->precedingSibling(), $preceding = array());
        return $preceding;
    }

    /**
     * Preceding Sibling
     * 
     * @access  public 
     * @return  array 
     */
    function &precedingSibling()
    {
        return Gadget_Tree::walkLink($this, 'preceding');
    }

    /**
     * Last Child
     * 
     * Example:
     * <code>
     *     if (!isset($child = &$node->lastChild()) {
     *         $child = &$node->addNewChild('one and only child');
     *     }
     *     $child->addNewChild('a new child of the last child');
     * </code>
     * 
     * @see     firstChild()
     * @access  public 
     * @return  object 
     */
    function &lastChild()
    {
        return $this->childs[$this->count() - 1];
    }

    /**
     * First Child
     * 
     * @see     lastChild()
     * @access  public 
     * @return  mixed 
     */
    function &firstChild()
    {
        return $this->childs[0];
    }

    /**
     * Count
     * 
     * @access  public 
     * @return  int 
     */
    function count()
    {
        return count($this->childs);
    }

    /**
     * Position
     * 
     * @access  public 
     * @return  int 
     */
    function position()
    {
        for ($i = 0, $node = &$this; $node = &$node->precending; $i++);
        return $i;
    }
    
    /**
     * Depth
     * 
     * @access  public
     * @return  int
     */
    function depth()
    {
        for ($i = 0, $node = &$this; $node = &$node->parent; $i++);
        return $i;
    }
    
    /**
     * Full Path
     * 
     * @access public 
     * @return string Returns the full path of the node.
     */
    function fullPath($separator = '/')
    {
        if (isset($this->fullPath)) {
            return $this->fullPath;
        }
        if (isset($this->preceding)) {
            return $this->fullPath = dirname($this->preceding->fullPath()) . 
                $separator . $this->name;
        }
        if (isset($this->parent)) {
            return $this->fullPath = $this->parent->fullPath() . 
                $separator . $this->name;
        }
        $this->fullPath = $this->name;
        foreach ($this->ancestor() as $node) {
            $this->fullPath = $node->name . $separator . $this->fullPath;
        }
        return $this->fullPath;
    }

}

?>
