<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Node.php,v 1.1 2005/04/04 11:51:13 mike Exp $
 */

/**
 * Requires Gadget_Tree_Node
 */
class_exists('Gadget_Tree_Node') or require_once 'Gadget/Tree/Node.php';

/**
 * Gadget_Tree_XML_Node
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_XML_Node extends Gadget_Tree_Node
{
    var $self = __CLASS__;
    var $namespace = '';
    var $isTextNode = false;
    
    /**
     * Constructor
     * 
     * @access  public
     */
    function Gadget_Tree_XML_Node($name)
    {
        if (empty($name)) {
            $this->isTextNode = true;
        } elseif (strstr($name, ':')) {
            list($this->namespace, $name) = explode(':', $name, 2);
        }
        parent::Gadget_Tree_Node($name);
    }
    
    /**
     * Set Meta
     * 
     * @access public 
     * @return mixed 
     */
    function setMeta($meta)
    {
        if ($this->isTextNode) {
            return Gadget_Tree::raiseError(
                'Cannot set meta information of a text node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        return parent::setMeta($meta);
    }

    /**
     * Add Meta
     * 
     * @access public 
     * @return mixed 
     */
    function addMeta($meta, $value = null)
    {
        if ($this->isTextNode) {
            return Gadget_Tree::raiseError(
                'Cannot add meta information to a text node.',
                GADGET_TREE_E_INVALID_OP
            );
        }
        return parent::addMeta($meta, $value);
    }

    /**
     * Attribute String
     * 
     * @access  public
     * @return  string
     */
    function attributeString()
    {
        if ($this->isTextNode) {
            return Gadget_Tree::raiseError(
                'A text node has no attributes',
                GADGET_TREE_INVALID_OP
            );
        }
        $attributes = '';
        foreach ($this->meta as $attr => $val){
            $attributes .= ' '. $attr .'="'. addCslashes($val, '"') .'"';
        }
        return $attributes;
    }
}
?>