<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Widget.php,v 1.2 2005/04/08 07:49:29 mike Exp $
 */

class_exists('Gadget') or require_once 'Gadget.php';
class_exists('Gadget_Object') or require_once 'Gadget/Object.php';

/** 
 * Gadget_Widget
 *
 * Base widget.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @since       1.0
 */
class Gadget_Widget extends Gadget_Object
{
    /**
     * Widget Queue ID
     * 
     * @var int
     */
    var $ID = -1;
    
    /**
     * HTML_QuickForm
     * 
     * @var object
     */
    var $qform;
    
    /**
     * Rendered Form
     * 
     * @var object
     */
    var $form;
    
    /**
     * Current Location
     * 
     * @var string
     */
    var $location;
    
    /**
     * Query String
     * 
     * @var string
     */
    var $queryString;
    
    /**
     * Whether user needs to authenticate
     * 
     * @var bool
     */
    var $auth = false;

    /**
     * Whether user needs special rights
     * 
     * @var mixed
     */
    var $rights = 0;

    /**
     * Whether to run main() at construction time
     * 
     * @var bool
     */
    var $main = true;
    
    /**
     * Whether to run render() at construction time
     * 
     * @var bool
     */
    var $render = false;

    /**
     * Whether to call render() on Gadget::finish()
     * 
     * @var bool
     */
    var $show = true;
    
    /**
     * Whether to call aggregateFormatter() at construction time
     * 
     * @var bool
     */
    var $aggregateFormatter = false;
    
    /**
     * Template
     * 
     * @var string
     */
    var $template;
    
    /**
     * Gadget Formelement Hooks
     * 
     * @var array
     */
    var $elements = array();
    
    /**
     * On Before Construct
     * 
     * @access  protected
     * @return  void
     */
    function onBeforeConstruct()
    {
        Gadget::queueWidget($this);
        
        $this->location = Gadget::location();
        
        // QueryString
        $this->queryString = &$_SERVER['QUERY_STRING'];
    }
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
        // Authentication required? 
        if ($this->auth && !Gadget::currentUser()) {
            Gadget::runService(SERVICE_AUTH);
        }
        
        // Sufficient Rights? 
        if ($this->rights) {
            if (!Gadget::runService(SERVICE_RIGHTS, $this->rights)) {
                $this->unsufficientRights();
            }
        }
        
        // log Widget
        Gadget::log('Widget', $this);
        
        // aggregate Formatter
        if ($this->aggregateFormatter) {
            $this->aggregateFormatter();
        }
        
        // call main()
        if ($this->main) {
            $this->main();
        }
        
        // call render()
        if ($this->render) {
            echo $this->render(true);
        }
    }
    
    /**
     * Main
     * 
     * @access  protected
     * @return  void
     */
    function main()
    {
        // void
    }
    
    /**
     * Attach QuickForm
     * 
     * @access  public
     * @return  object
     * @param   string
     * @param   array
     */
    function &form($formName, $attributes = array(), $location = null, $method = 'post')
    {
        isset($location) or $location = $_SERVER['REQUEST_URI'];
        class_exists('HTML_QuickForm') or require_once 'HTML/QuickForm.php';
        $this->qform = &new HTML_QuickForm(
            $formName, $method, $location, '_self', $attributes, true);
        $this->qform->setJsWarnings('','');
        return $this->qform;
    }
    
    /**
     * Render
     * 
     * @access  public
     * @return  string
     */
    function render($finish = false)
    {
        $finish && $this->show = false;
        return Gadget::runService(SERVICE_RENDER, $this, true);
    }

    /**
     * Odd Even
     * 
     * @access  public
     * @return  string
     * @param   int
     */
    function oddEven($int)
    {
        return $int % 2 ? 'odd' : 'even';
    }
    
    /**
     * Load Template
     * 
     * @access  public
     * @return  string
     * @param   string
     * @param   bool
     */
    function load($template, $render = false)
    {
        strpos($template, '.') or $template .= '.html';
        if ($render) {
            $widget = clone($this);
            $widget->template = $template;
            return $widget->render(true);
        } else {
            ob_start();
            include Gadget::getConf('Gadget', 'tpldir') .'/'. $template;
            return ob_get_clean();
        }
    }
    
    /**
     * Aggregate Formatter
     * 
     * @access  protected
     * @return  void
     */
    function aggregateFormatter()
    {
        class_exists('Gadget_Formatter') or require_once 'Gadget/Formatter.php';
        if (PHP_VERSION < 5) {
            aggregate($this, 'Gadget_Formatter');
        } elseif (empty($GLOBALS['_Gadget']['Widget']['aggregatedFormatter'])) {
            $GLOBALS['_Gadget']['Widget']['aggregatedFormatter'] = true;
            classkit_aggregate_methods('Gadget_Widget', 'Gadget_Formatter');
        }
    }
    
    /**
     * Unsufficient Rights Hook
     * 
     * @access  protected
     * @return  void
     */
    function unsufficientRights()
    {
        Gadget::newWidget(array('template' => 'unsufficientRights.html'));
        $this->render = false;
        $this->show = false;
    }
    
}

?>
