<?php

/**
 * Gadget Web Development Kit
 * 
 * The Gadget WDK empowers you with an thight integrated infrastructure for web 
 * development.  It is based on Widgets and borrows most of its power from PEAR 
 * by using HTML_QuickForm, I18Nv2, several HTTP packages and DB_DataObject 
 * resp. DB (see dependencies).  Many important services like authentication, 
 * rights management and error handling as well as providers for "current user",
 * preferences and configuration can be instantly replaced.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tools
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Gadget.php,v 1.1 2005/04/04 12:15:54 mike Exp $
 */

/**
 * HTML_QuickForm_Renderer_Gadget
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tools
 * @since       1.0
 */
class HTML_QuickForm_Renderer_Gadget
{
    var $form;
    var $currentGroup;
    
   /**
    * Start Form
    */
    function startForm(&$form)
    {
        $this->form = new HTML_QuickForm_Renderer_Gadget_Form;
        $this->currentGroup = null;
    }

   /**
    * Finish Form
    */
    function finishForm(&$form)
    {
        $this->form->attributes = $form->getAttributes(true);
        $this->form->javascript = $form->getValidationScript();
        $this->currentGroup = null;
    }

   /**
    * Render Header
    */
    function renderHeader(&$header)
    {
        $this->form->header->{$header->getName()} = $header->toHtml();
    }

   /**
    * Render Element
    */
    function renderElement(&$element, $required, $error)
    {
        $name = $element->getName();
        $html = $element->toHtml();
        
        $elmt = (object) array(
            'name'      => $name,
            'id'        => $element->getAttribute('id'),
            'label'     => $element->getLabel(),
            'html'      => $html,
            'frozen'    => $element->isFrozen(),
            'required'  => $required,
            'error'     => $error,
            'value'     => $element->getValue(),
        );
        
        if ($this->currentGroup) {
            $this->form->{$this->currentGroup}->elements[] = $elmt;
        }
        
        if ($name != $this->currentGroup) {
            if (preg_match('/^(.*)\[(.*)\]$/', $name, $m)) {
                $name = $m[1];
                if (strlen($m[2])) {
                    $this->form->{$name}[$m[2]] = $elmt;
                } else {
                    $this->form->{$name}[] = $elmt;
                }
            } else {
                $this->form->$name = $elmt;
            }
        }
    }

   /**
    * Render Hidden
    */
    function renderHidden(&$element)
    {
        $this->form->hidden .= "\n". $element->toHtml();
    }

   /**
    * Render HTML
    */
    function renderHtml(&$data)
    {
        return;
    }

   /**
    * Start Group
    */
    function startGroup(&$group, $required, $error)
    {
        $name = $group->getName();
        $this->form->$name = (object) array(
            'name'      => $name,
            'label'     => $group->getLabel(),
            'html'      => '',
            'frozen'    => $group->isFrozen(),
            'required'  => $required,
            'error'     => $error,
            'elements'  => array()
        );
        $this->currentGroup = $name;
    }

   /**
    * Finish Group
    */
    function finishGroup(&$group)
    {
        $name = $group->getName();
        $html = array();
        foreach ($this->form->$name->elements as $elmt) {
            $html[] = $elmt->html;
        }
        $this->form->$name->html = implode($group->_separator, $html);
        $this->currentGroup = null;
    }
}

class HTML_QuickForm_Renderer_Gadget_Form
{
    var $header;
    var $hidden = '';
    var $javasript = '';
    var $attributes = '';
    
    function HTML_QuickForm_Renderer_Gadget_Form()
    {
        $this->header = new StdClass;
    }
    
    function element($element)
    {
        return $this->label($element, true) . $this->html($element) . $this->error($element, true);
    }
    
    function label($element, $wrap = false, $prepend = '', $append = '<br />', $insert = '')
    {
        if ($wrap) {
            return $prepend . sprintf('<label %s for="%s">%s</label>', $insert, $element, $this->$element->label) . $append;
        }
        return $this->$element->label;
    }
    
    function html($element)
    {
        return $this->$element->html;
    }
    
    function error($element, $wrap = false, $prepend = '<br />', $append = '', $insert = 'class="error"')
    {
        if (!$this->$element->error) {
            return '';
        }
        if ($wrap) {
            return $prepend . sprintf('<span %s>%s</span>', $insert, $this->$element->error) . $append;
        }
        return $this->$element->error;
    }
    
    function has($element)
    {
        return isset($this->$element);
    }
}
?>
