<?php

/**
 * Gadget Web Development Kit
 * 
 * The Gadget WDK empowers you with an thight integrated infrastructure for web 
 * development.  It is based on Widgets and borrows most of its power from PEAR 
 * by using HTML_QuickForm, I18Nv2, several HTTP packages and DB_DataObject 
 * resp. DB (see dependencies).  Many important services like authentication, 
 * rights management and error handling as well as providers for "current user",
 * preferences and configuration can be instantly replaced.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tools
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Date.php,v 1.1 2005/04/04 12:15:55 mike Exp $
 */

/**
 * Requires HTML_QuickForm
 */
class_exists('HTML_QuickForm') or require_once 'HTML/QuickForm.php';

/**
 * Requires HTML_QuickForm_Rule
 */
class_exists('HTML_QuickForm_Rule') or require_once 'HTML/QuickForm/Rule.php';

/**
 * Register Rule
 */
HTML_QuickForm::registerRule('date', null, 'HTML_QuickForm_Rule_Date', 'HTML/QuickForm/Rule/Date.php');

/**
 * HTML_QuickForm_Rule_Date
 *
 * @package     Gadget
 * @subpackage  Gadget_Tools
 * @since       1.0
 */
class HTML_QuickForm_Rule_Date extends HTML_QuickForm_Rule
{
    /**
     * Validate
     * 
     * @access  public
     * @return  bool
     * @param   array
     * @param   array
     */
    function validate($date, $order = null)
    {
        class_exists('Validate') or require_once 'Validate.php';

        if (!$order) {
            list($d, $m, $y) = array_values($date);
        } else {
            list(${$order[0]}, ${$order[1]}, ${$order[2]}) = array_values($date);
        }
        
        return Validate::date(
            sprintf('%02d.%02d.%d', $d, $m, $y), 
            array('format' => '%d.%m.%Y')
        );
    }
    
    /**
     * Gadget: add Date
     * 
     * @access  public
     * @return  void
     * @param   object  $form
     */
    function addDate(&$form, $name, $label, $dmin = -5, $dmax = -100)
    {
        $theyear = date('Y');
        $options = array(
            'language'  => Gadget::getPref('language'),
            'format'    => 'd F Y',
            'minYear'   => $theyear + $dmin,
            'maxYear'   => $theyear + $dmax,
        );
        $form->addElement('date', $name, $label, $options);
        $form->addRule($name, _('Bitte ein gltiges Datum eingeben!'), 'date');
    }
    
    /**
     * Gadget: fix Date
     * 
     * @access  public
     * @return  void
     * @param   array   $date
     */
    function fixDate(&$date, $format = '%d-%02d-%02d')
    {
        $temp = sprintf($format, $date['Y'], $date['F'], $date['d']);
        $date = $temp;
    }
    
    /**
     * Gadget: set Date Default
     * 
     * @access  public
     * @return  void
     * @param   object  $form
     * @param   string  $birthday
     */
    function defDate(&$form, $name, $date, $sep = '-')
    {
        list($d['Y'], $d['F'], $d['d']) = is_array($date) ?
            array_values($date) : explode($sep, substr($date, 0, 8));
        $form->setDefaults(array($name => $d));
    }
    
}

?>
